/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef plarena_h___^M
#define plarena_h___^M
/*^M
 * Lifetime-based fast allocation, inspired by much prior art, including^M
 * "Fast Allocation and Deallocation of Memory Based on Object Lifetimes"^M
 * David R. Hanson, Software -- Practice and Experience, Vol. 20(1).^M
 *^M
 * Also supports LIFO allocation (PL_ARENA_MARK/PL_ARENA_RELEASE).^M
 */^M
#include "prtypes.h"^M
#include "plarenas.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
typedef struct PLArena          PLArena;^M
^M
struct PLArena {^M
    PLArena     *next;          /* next arena for this lifetime */^M
    PRUword     base;           /* aligned base address, follows this header */^M
    PRUword     limit;          /* one beyond last byte in arena */^M
    PRUword     avail;          /* points to next available byte */^M
};^M
^M
#ifdef PL_ARENAMETER^M
typedef struct PLArenaStats PLArenaStats;^M
^M
struct PLArenaStats {^M
    PLArenaStats  *next;        /* next in arenaStats list */^M
    char          *name;        /* name for debugging */^M
    PRUint32      narenas;      /* number of arenas in pool */^M
    PRUint32      nallocs;      /* number of PL_ARENA_ALLOCATE() calls */^M
    PRUint32      nreclaims;    /* number of reclaims from freeArenas */^M
    PRUint32      nmallocs;     /* number of malloc() calls */^M
    PRUint32      ndeallocs;    /* number of lifetime deallocations */^M
    PRUint32      ngrows;       /* number of PL_ARENA_GROW() calls */^M
    PRUint32      ninplace;     /* number of in-place growths */^M
    PRUint32      nreleases;    /* number of PL_ARENA_RELEASE() calls */^M
    PRUint32      nfastrels;    /* number of "fast path" releases */^M
    PRUint32      nbytes;       /* total bytes allocated */^M
    PRUint32      maxalloc;     /* maximum allocation size in bytes */^M
    PRFloat64     variance;     /* size variance accumulator */^M
};^M
#endif^M
^M
struct PLArenaPool {^M
    PLArena     first;          /* first arena in pool list */^M
    PLArena     *current;       /* arena from which to allocate space */^M
    PRUint32    arenasize;      /* net exact size of a new arena */^M
    PRUword     mask;           /* alignment mask (power-of-2 - 1) */^M
#ifdef PL_ARENAMETER^M
    PLArenaStats stats;^M
#endif^M
};^M
^M
/*^M
 * If the including .c file uses only one power-of-2 alignment, it may define^M
 * PL_ARENA_CONST_ALIGN_MASK to the alignment mask and save a few instructions^M
 * per ALLOCATE and GROW.^M
 */^M
#ifdef PL_ARENA_CONST_ALIGN_MASK^M
#define PL_ARENA_ALIGN(pool, n) (((PRUword)(n) + PL_ARENA_CONST_ALIGN_MASK) \^M
                                & ~PL_ARENA_CONST_ALIGN_MASK)^M
^M
#define PL_INIT_ARENA_POOL(pool, name, size) \^M
        PL_InitArenaPool(pool, name, size, PL_ARENA_CONST_ALIGN_MASK + 1)^M
#else^M
#define PL_ARENA_ALIGN(pool, n) (((PRUword)(n) + (pool)->mask) & ~(pool)->mask)^M
#endif^M
^M
#define PL_ARENA_ALLOCATE(p, pool, nb) \^M
    PR_BEGIN_MACRO \^M
        PLArena *_a = (pool)->current; \^M
        PRUint32 _nb = PL_ARENA_ALIGN(pool, nb); \^M
        PRUword _p = _a->avail; \^M
        PRUword _q = _p + _nb; \^M
        if (_q > _a->limit) \^M
            _p = (PRUword)PL_ArenaAllocate(pool, _nb); \^M
        else \^M
            _a->avail = _q; \^M
        p = (void *)_p; \^M
        PL_ArenaCountAllocation(pool, nb); \^M
    PR_END_MACRO^M
^M
#define PL_ARENA_GROW(p, pool, size, incr) \^M
    PR_BEGIN_MACRO \^M
        PLArena *_a = (pool)->current; \^M
        PRUint32 _incr = PL_ARENA_ALIGN(pool, incr); \^M
        PRUword _p = _a->avail; \^M
        PRUword _q = _p + _incr; \^M
        if (_p == (PRUword)(p) + PL_ARENA_ALIGN(pool, size) && \^M
            _q <= _a->limit) { \^M
            _a->avail = _q; \^M
            PL_ArenaCountInplaceGrowth(pool, size, incr); \^M
        } else { \^M
            p = PL_ArenaGrow(pool, p, size, incr); \^M
        } \^M
        PL_ArenaCountGrowth(pool, size, incr); \^M
    PR_END_MACRO^M
^M
#define PL_ARENA_MARK(pool) ((void *) (pool)->current->avail)^M
#define PR_UPTRDIFF(p,q) ((PRUword)(p) - (PRUword)(q))^M
^M
#ifdef DEBUG^M
#define PL_FREE_PATTERN 0xDA^M
#define PL_CLEAR_UNUSED(a) (PR_ASSERT((a)->avail <= (a)->limit), \^M
                           memset((void*)(a)->avail, PL_FREE_PATTERN, \^M
                           (a)->limit - (a)->avail))^M
#define PL_CLEAR_ARENA(a)  memset((void*)(a), PL_FREE_PATTERN, \^M
                           (a)->limit - (PRUword)(a))^M
#else^M
#define PL_CLEAR_UNUSED(a)^M
#define PL_CLEAR_ARENA(a)^M
#endif^M
^M
#define PL_ARENA_RELEASE(pool, mark) \^M
    PR_BEGIN_MACRO \^M
        char *_m = (char *)(mark); \^M
        PLArena *_a = (pool)->current; \^M
        if (PR_UPTRDIFF(_m, _a->base) <= PR_UPTRDIFF(_a->avail, _a->base)) { \^M
            _a->avail = (PRUword)PL_ARENA_ALIGN(pool, _m); \^M
            PL_CLEAR_UNUSED(_a); \^M
            PL_ArenaCountRetract(pool, _m); \^M
        } else { \^M
            PL_ArenaRelease(pool, _m); \^M
        } \^M
        PL_ArenaCountRelease(pool, _m); \^M
    PR_END_MACRO^M
^M
#ifdef PL_ARENAMETER^M
#define PL_COUNT_ARENA(pool,op) ((pool)->stats.narenas op)^M
#else^M
#define PL_COUNT_ARENA(pool,op)^M
#endif^M
^M
#define PL_ARENA_DESTROY(pool, a, pnext) \^M
    PR_BEGIN_MACRO \^M
        PL_COUNT_ARENA(pool,--); \^M
        if ((pool)->current == (a)) (pool)->current = &(pool)->first; \^M
        *(pnext) = (a)->next; \^M
        PL_CLEAR_ARENA(a); \^M
        free(a); \^M
        (a) = 0; \^M
    PR_END_MACRO^M
^M
#ifdef PL_ARENAMETER^M
^M
#include <stdio.h>^M
^M
PR_EXTERN(void) PL_ArenaCountAllocation(PLArenaPool *pool, PRUint32 nb);^M
^M
PR_EXTERN(void) PL_ArenaCountInplaceGrowth(^M
    PLArenaPool *pool, PRUint32 size, PRUint32 incr);^M
^M
PR_EXTERN(void) PL_ArenaCountGrowth(^M
    PLArenaPool *pool, PRUint32 size, PRUint32 incr);^M
^M
PR_EXTERN(void) PL_ArenaCountRelease(PLArenaPool *pool, char *mark);^M
^M
PR_EXTERN(void) PL_ArenaCountRetract(PLArenaPool *pool, char *mark);^M
^M
PR_EXTERN(void) PL_DumpArenaStats(FILE *fp);^M
^M
#else  /* !PL_ARENAMETER */^M
^M
#define PL_ArenaCountAllocation(ap, nb)                 /* nothing */^M
#define PL_ArenaCountInplaceGrowth(ap, size, incr)      /* nothing */^M
#define PL_ArenaCountGrowth(ap, size, incr)             /* nothing */^M
#define PL_ArenaCountRelease(ap, mark)                  /* nothing */^M
#define PL_ArenaCountRetract(ap, mark)                  /* nothing */^M
^M
#endif /* !PL_ARENAMETER */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* plarena_h___ */^M
