/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/* GLOBAL FUNCTIONS:^M
** DESCRIPTION:^M
**     PR Atomic operations^M
*/^M
^M
#ifndef pratom_h___^M
#define pratom_h___^M
^M
#include "prtypes.h"^M
#include "prlock.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** FUNCTION: PR_AtomicIncrement^M
** DESCRIPTION:^M
**    Atomically increment a 32 bit value.^M
** INPUTS:^M
**    val:  a pointer to the value to increment^M
** RETURN:^M
**    the returned value is the result of the increment^M
*/^M
NSPR_API(PRInt32)	PR_AtomicIncrement(PRInt32 *val);^M
^M
/*^M
** FUNCTION: PR_AtomicDecrement^M
** DESCRIPTION:^M
**    Atomically decrement a 32 bit value.^M
** INPUTS:^M
**    val:  a pointer to the value to decrement^M
** RETURN:^M
**    the returned value is the result of the decrement^M
*/^M
NSPR_API(PRInt32)	PR_AtomicDecrement(PRInt32 *val);^M
^M
/*^M
** FUNCTION: PR_AtomicSet^M
** DESCRIPTION:^M
**    Atomically set a 32 bit value.^M
** INPUTS:^M
**    val: A pointer to a 32 bit value to be set^M
**    newval: The newvalue to assign to val^M
** RETURN:^M
**    Returns the prior value^M
*/^M
NSPR_API(PRInt32) PR_AtomicSet(PRInt32 *val, PRInt32 newval);^M
^M
/*^M
** FUNCTION: PR_AtomicAdd^M
** DESCRIPTION:^M
**    Atomically add a 32 bit value.^M
** INPUTS:^M
**    ptr:  a pointer to the value to increment^M
**	  val:  value to be added^M
** RETURN:^M
**    the returned value is the result of the addition^M
*/^M
NSPR_API(PRInt32)	PR_AtomicAdd(PRInt32 *ptr, PRInt32 val);^M
^M
/*^M
** LIFO linked-list (stack)^M
*/^M
typedef struct PRStackElemStr PRStackElem;^M
^M
struct PRStackElemStr {^M
    PRStackElem	*prstk_elem_next;	/* next pointer MUST be at offset 0;^M
									  assembly language code relies on this */^M
};^M
^M
typedef struct PRStackStr PRStack;^M
^M
/*^M
** FUNCTION: PR_CreateStack^M
** DESCRIPTION:^M
**    Create a stack, a LIFO linked list^M
** INPUTS:^M
**    stack_name:  a pointer to string containing the name of the stack^M
** RETURN:^M
**    A pointer to the created stack, if successful, else NULL.^M
*/^M
NSPR_API(PRStack *)	PR_CreateStack(const char *stack_name);^M
^M
/*^M
** FUNCTION: PR_StackPush^M
** DESCRIPTION:^M
**    Push an element on the top of the stack^M
** INPUTS:^M
**    stack:		pointer to the stack^M
**    stack_elem:	pointer to the stack element^M
** RETURN:^M
**    None^M
*/^M
NSPR_API(void)			PR_StackPush(PRStack *stack, PRStackElem *stack_elem);^M
^M
/*^M
** FUNCTION: PR_StackPop^M
** DESCRIPTION:^M
**    Remove the element on the top of the stack^M
** INPUTS:^M
**    stack:		pointer to the stack^M
** RETURN:^M
**    A pointer to the stack element removed from the top of the stack,^M
**	  if non-empty,^M
**    else NULL^M
*/^M
NSPR_API(PRStackElem *)	PR_StackPop(PRStack *stack);^M
^M
/*^M
** FUNCTION: PR_DestroyStack^M
** DESCRIPTION:^M
**    Destroy the stack^M
** INPUTS:^M
**    stack:		pointer to the stack^M
** RETURN:^M
**    PR_SUCCESS - if successfully deleted^M
**	  PR_FAILURE - if the stack is not empty^M
**					PR_GetError will return^M
**						PR_INVALID_STATE_ERROR - stack is not empty^M
*/^M
NSPR_API(PRStatus)		PR_DestroyStack(PRStack *stack);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* pratom_h___ */^M
