/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prcountr_h___^M
#define prcountr_h___^M
^M
/*----------------------------------------------------------------------------^M
** prcountr.h -- NSPR Instrumentation counters^M
**^M
** The NSPR Counter Feature provides a means to "count^M
** something." Counters can be dynamically defined, incremented,^M
** decremented, set, and deleted under application program^M
** control.^M
** 																                   ^M
** The Counter Feature is intended to be used as instrumentation,                  ^M
** not as operational data. If you need a counter for operational                  ^M
** data, use native integral types.                                                ^M
** 																                   ^M
** Counters are 32bit unsigned intergers. On overflow, a counter                   ^M
** will wrap. No exception is recognized or reported.                              ^M
**                                                                                 ^M
** A counter can be dynamically created using a two level naming^M
** convention. A "handle" is returned when the counter is^M
** created. The counter can subsequently be addressed by its^M
** handle. An API is provided to get an existing counter's handle^M
** given the names with  which it was originally created. ^M
** Similarly, a counter's name can be retrieved given its handle.^M
** ^M
** The counter naming convention is a two-level hierarchy. The^M
** QName is the higher level of the hierarchy; RName is the^M
** lower level. RNames can be thought of as existing within a^M
** QName. The same RName can exist within multiple QNames. QNames^M
** are unique. The NSPR Counter is not a near-zero overhead^M
** feature. Application designers should be aware of ^M
** serialization issues when using the Counter API. Creating a^M
** counter locks a large asset, potentially causing a stall. This^M
** suggest that applications should create counters at component^M
** initialization, for example, and not create and destroy them^M
** willy-nilly. ... You have been warned.^M
** ^M
** Incrementing and Adding to counters uses atomic operations.^M
** The performance of these operations will vary from platform^M
** to platform. On platforms where atomic operations are not^M
** supported the overhead may be substantial.^M
** ^M
** When traversing the counter database with FindNext functions,^M
** the instantaneous values of any given counter is that at the^M
** moment of extraction. The state of the entire counter database^M
** may not be viewed as atomic.^M
** ^M
** The counter interface may be disabled (No-Op'd) at compile^M
** time. When DEBUG is defined at compile time, the Counter^M
** Feature is compiled into NSPR and applications invoking it.^M
** When DEBUG is not defined, the counter macros compile to^M
** nothing. To force the Counter Feature to be compiled into an^M
** optimized build, define FORCE_NSPR_COUNTERS at compile time^M
** for both NSPR and the application intending to use it.^M
** ^M
** Application designers should use the macro form of the Counter^M
** Feature methods to minimize performance impact in optimized^M
** builds. The macros normally compile to nothing on optimized^M
** builds.^M
** ^M
** Application designers should be aware of the effects of^M
** debug and optimized build differences when using result of the^M
** Counter Feature macros in expressions.^M
** ^M
** The Counter Feature is thread-safe and SMP safe.^M
** ^M
** /lth. 09-Jun-1998.^M
*/^M
^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** Opaque counter handle type.^M
** ... don't even think of looking in here.^M
**^M
*/^M
typedef void *  PRCounterHandle;^M
^M
#define PRCOUNTER_NAME_MAX 31^M
#define PRCOUNTER_DESC_MAX 255^M
^M
^M
#if defined(DEBUG) || defined(FORCE_NSPR_COUNTERS)^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_DEFINE_COUNTER() -- Define a PRCounterHandle^M
** ^M
** DESCRIPTION: PR_DEFINE_COUNTER() is used to define a counter^M
** handle.^M
** ^M
*/^M
#define PR_DEFINE_COUNTER(name) PRCounterHandle name^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_INIT_COUNTER_HANDLE() -- Set the value of a PRCounterHandle^M
** ^M
** DESCRIPTION: ^M
** PR_INIT_COUNTER_HANDLE() sets the value of a PRCounterHandle^M
** to value.^M
** ^M
*/^M
#define PR_INIT_COUNTER_HANDLE(handle,value)\^M
    (handle) = (PRCounterHandle)(value)^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_CreateCounter() -- Create a counter^M
** ^M
** DESCRIPTION: PR_CreateCounter() creates a counter object and^M
** initializes it to zero.^M
** ^M
** The macro form takes as its first argument the name of the^M
** PRCounterHandle to receive the handle returned from^M
** PR_CreateCounter().^M
** ^M
** INPUTS:^M
**  qName: The QName for the counter object. The maximum length^M
** of qName is defined by PRCOUNTER_NAME_MAX^M
** ^M
**  rName: The RName for the counter object. The maximum length^M
** of qName is defined by PRCOUNTER_NAME_MAX^M
** ^M
**  descrioption: The description of the counter object. The^M
** maximum length of description is defined by^M
** PRCOUNTER_DESC_MAX.^M
** ^M
** OUTPUTS:^M
** ^M
** RETURNS:^M
**  PRCounterHandle.^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_CREATE_COUNTER(handle,qName,rName,description)\^M
   (handle) = PR_CreateCounter((qName),(rName),(description))^M
^M
NSPR_API(PRCounterHandle) ^M
	PR_CreateCounter( ^M
		const char *qName, ^M
    	const char *rName, ^M
        const char *description ^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_DestroyCounter() -- Destroy a counter object.^M
** ^M
** DESCRIPTION: PR_DestroyCounter() removes a counter and^M
** unregisters its handle from the counter database.^M
** ^M
** INPUTS:^M
**  handle: the PRCounterHandle of the counter to be destroyed.^M
** ^M
** OUTPUTS: ^M
**  The counter is destroyed.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_DESTROY_COUNTER(handle) PR_DestroyCounter((handle))^M
^M
NSPR_API(void) ^M
	PR_DestroyCounter( ^M
		PRCounterHandle handle ^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_GetCounterHandleFromName() -- Retreive a^M
** counter's handle give its name.^M
** ^M
** DESCRIPTION: PR_GetCounterHandleFromName() retreives a^M
** counter's handle from the counter database, given the name^M
** the counter was originally created with.^M
** ^M
** INPUTS:^M
**  qName: Counter's original QName.^M
**  rName: Counter's original RName.^M
** ^M
** OUTPUTS:^M
** ^M
** RETURNS: ^M
**  PRCounterHandle or PRCounterError.^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_GET_COUNTER_HANDLE_FROM_NAME(handle,qName,rName)\^M
    (handle) = PR_GetCounterHandleFromName((qName),(rName))^M
^M
NSPR_API(PRCounterHandle) ^M
	PR_GetCounterHandleFromName( ^M
    	const char *qName, ^M
    	const char *rName ^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_GetCounterNameFromHandle() -- Retreive a^M
** counter's name, given its handle.^M
** ^M
** DESCRIPTION: PR_GetCounterNameFromHandle() retreives a^M
** counter's name given its handle.^M
** ^M
** INPUTS:^M
**  qName: Where to store a pointer to qName.^M
**  rName: Where to store a pointer to rName.^M
**  description: Where to store a pointer to description.^M
** ^M
** OUTPUTS: Pointers to the Counter Feature's copies of the names^M
** used when the counters were created.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_GET_COUNTER_NAME_FROM_HANDLE(handle,qName,rName,description)\^M
    PR_GetCounterNameFromHandle((handle),(qName),(rName),(description))^M
^M
NSPR_API(void) ^M
	PR_GetCounterNameFromHandle( ^M
    	PRCounterHandle handle,  ^M
	    const char **qName, ^M
	    const char **rName, ^M
		const char **description ^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_IncrementCounter() -- Add one to the referenced^M
** counter.^M
** ^M
** DESCRIPTION: Add one to the referenced counter.^M
** ^M
** INPUTS:^M
**  handle: The PRCounterHandle of the counter to be incremented^M
** ^M
** OUTPUTS: The counter is incrementd.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_INCREMENT_COUNTER(handle) PR_IncrementCounter(handle)^M
^M
NSPR_API(void) ^M
	PR_IncrementCounter( ^M
		PRCounterHandle handle^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_DecrementCounter() -- Subtract one from the^M
** referenced counter^M
** ^M
** DESCRIPTION: Subtract one from the referenced counter.^M
** ^M
** INPUTS: ^M
**  handle: The PRCounterHandle of the coutner to be^M
** decremented.^M
** ^M
** OUTPUTS: the counter is decremented.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_DECREMENT_COUNTER(handle) PR_DecrementCounter(handle)^M
^M
NSPR_API(void) ^M
	PR_DecrementCounter( ^M
		PRCounterHandle handle^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_AddToCounter() -- Add a value to a counter.^M
** ^M
** DESCRIPTION: Add value to the counter referenced by handle.^M
** ^M
** INPUTS:^M
**  handle: the PRCounterHandle of the counter to be added to.^M
** ^M
**  value: the value to be added to the counter.^M
** ^M
** OUTPUTS: new value for counter.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_ADD_TO_COUNTER(handle,value)\^M
    PR_AddToCounter((handle),(value))^M
^M
NSPR_API(void) ^M
	PR_AddToCounter( ^M
    	PRCounterHandle handle, ^M
	    PRUint32 value ^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_SubtractFromCounter() -- A value is subtracted^M
** from a counter.^M
** ^M
** DESCRIPTION:^M
** Subtract a value from a counter.^M
** ^M
** INPUTS:^M
**  handle: the PRCounterHandle of the counter to be subtracted^M
** from.^M
** ^M
**  value: the value to be subtracted from the counter.^M
** ^M
** OUTPUTS: new value for counter^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_SUBTRACT_FROM_COUNTER(handle,value)\^M
    PR_SubtractFromCounter((handle),(value))^M
^M
NSPR_API(void) ^M
	PR_SubtractFromCounter( ^M
    	PRCounterHandle handle, ^M
	    PRUint32 value ^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_GetCounter() -- Retreive the value of a counter^M
** ^M
** DESCRIPTION:^M
** Retreive the value of a counter.^M
** ^M
** INPUTS:^M
**  handle: the PR_CounterHandle of the counter to be retreived^M
** ^M
** OUTPUTS:^M
** ^M
** RETURNS: The value of the referenced counter^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_GET_COUNTER(counter,handle)\^M
    (counter) = PR_GetCounter((handle))^M
^M
NSPR_API(PRUint32) ^M
	PR_GetCounter( ^M
		PRCounterHandle handle ^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_SetCounter() -- Replace the content of counter^M
** with value.^M
** ^M
** DESCRIPTION: The contents of the referenced counter are^M
** replaced by value.^M
** ^M
** INPUTS:^M
**  handle: the PRCounterHandle of the counter whose contents^M
** are to be replaced.^M
** ^M
**  value: the new value of the counter.^M
** ^M
** OUTPUTS:^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_SET_COUNTER(handle,value) PR_SetCounter((handle),(value))^M
^M
NSPR_API(void) ^M
	PR_SetCounter( ^M
		PRCounterHandle handle, ^M
		PRUint32 value ^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_FindNextCounterQname() -- Retreive the next QName counter^M
** handle iterator^M
** ^M
** DESCRIPTION:^M
** PR_FindNextCounterQname() retreives the first or next Qname^M
** the counter data base, depending on the value of handle. When^M
** handle is NULL, the function attempts to retreive the first^M
** QName handle in the database. When handle is a handle previosly^M
** retreived QName handle, then the function attempts to retreive^M
** the next QName handle.^M
** ^M
** INPUTS: ^M
**  handle: PRCounterHandle or NULL.^M
** ^M
** OUTPUTS: returned^M
** ^M
** RETURNS: PRCounterHandle or NULL when no more QName counter^M
** handles are present.^M
** ^M
** RESTRICTIONS:^M
**  A concurrent PR_CreateCounter() or PR_DestroyCounter() may^M
** cause unpredictable results.^M
** ^M
** A PRCounterHandle returned from this function may only be used^M
** in another PR_FindNextCounterQname() function call; other^M
** operations may cause unpredictable results.^M
** ^M
*/^M
#define PR_FIND_NEXT_COUNTER_QNAME(next,handle)\^M
    (next) = PR_FindNextCounterQname((handle))^M
^M
NSPR_API(PRCounterHandle) ^M
	PR_FindNextCounterQname( ^M
        PRCounterHandle handle^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_FindNextCounterRname() -- Retreive the next RName counter^M
** handle iterator^M
** ^M
** DESCRIPTION:^M
** PR_FindNextCounterRname() retreives the first or next RNname^M
** handle from the counter data base, depending on the^M
** value of handle. When handle is NULL, the function attempts to^M
** retreive the first RName handle in the database. When handle is^M
** a handle previosly retreived RName handle, then the function^M
** attempts to retreive the next RName handle.^M
** ^M
** INPUTS:^M
**  handle: PRCounterHandle or NULL.^M
**  qhandle: PRCounterHandle of a previously aquired via^M
** PR_FIND_NEXT_QNAME_HANDLE()^M
** ^M
** OUTPUTS: returned^M
** ^M
** RETURNS: PRCounterHandle or NULL when no more RName counter^M
** handles are present.^M
** ^M
** RESTRICTIONS:^M
**  A concurrent PR_CreateCounter() or PR_DestroyCounter() may^M
** cause unpredictable results.^M
** ^M
** A PRCounterHandle returned from this function may only be used^M
** in another PR_FindNextCounterRname() function call; other^M
** operations may cause unpredictable results.^M
** ^M
*/^M
#define PR_FIND_NEXT_COUNTER_RNAME(next,rhandle,qhandle)\^M
    (next) = PR_FindNextCounterRname((rhandle),(qhandle))^M
^M
NSPR_API(PRCounterHandle) ^M
	PR_FindNextCounterRname( ^M
        PRCounterHandle rhandle,^M
        PRCounterHandle qhandle^M
);^M
^M
^M
#else /* ( !(defined(DEBUG) || defined(FORCE_NSPR_COUNTERS)) */^M
/*^M
** When counters are not compiled in, provide macros that ^M
** evaluate to No-Ops.^M
**^M
*/^M
^M
#define PR_DEFINE_COUNTER(name) PRCounterHandle name^M
#define PR_INIT_COUNTER_HANDLE(handle,value)^M
#define PR_CREATE_COUNTER(handle,qName,rName,description)^M
#define PR_DESTROY_COUNTER(handle)^M
#define PR_GET_COUNTER_HANDLE_FROM_NAME(handle,qName,rName)^M
#define PR_GET_COUNTER_NAME_FROM_HANDLE(handle,qName,rName,description )^M
#define PR_INCREMENT_COUNTER(handle)^M
#define PR_DECREMENT_COUNTER(handle)^M
#define PR_ADD_TO_COUNTER(handle,value)^M
#define PR_SUBTRACT_FROM_COUNTER(handle,value)^M
#define PR_GET_COUNTER(counter,handle) 0^M
#define PR_SET_COUNTER(handle,value)^M
#define PR_FIND_NEXT_COUNTER_QNAME(next,handle) NULL^M
#define PR_FIND_NEXT_COUNTER_RNAME(next,rhandle,qhandle)^M
^M
#endif /* ( !(defined(DEBUG) || defined(FORCE_NSPR_COUNTERS)) */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prcountr_h___ */^M
