/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions requiored by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prenv_h___^M
#define prenv_h___^M
^M
#include "prtypes.h"^M
^M
/*******************************************************************************/^M
/*******************************************************************************/^M
/****************** THESE FUNCTIONS MAY NOT BE THREAD SAFE *********************/^M
/*******************************************************************************/^M
/*******************************************************************************/^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** PR_GetEnv() -- Retrieve value of environment variable^M
** ^M
** Description:^M
** PR_GetEnv() is modeled on Unix getenv().^M
** ^M
** ^M
** Inputs: ^M
**   var -- The name of the environment variable^M
** ^M
** Returns:^M
**   The value of the environment variable 'var' or NULL if^M
** the variable is undefined.^M
** ^M
** Restrictions:^M
**   You'd think that a POSIX getenv(), putenv() would be^M
**   consistently implemented everywhere. Surprise! It is not. On^M
**   some platforms, a putenv() where the argument is of^M
**   the form "name"  causes the named environment variable to^M
**   be un-set; that is: a subsequent getenv() returns NULL. On^M
**   other platforms, the putenv() fails, on others, it is a^M
**   no-op. Similarly, a putenv() where the argument is of the^M
**   form "name=" causes the named environment variable to be^M
**   un-set; a subsequent call to getenv() returns NULL. On^M
**   other platforms, a subsequent call to getenv() returns a^M
**   pointer to a null-string (a byte of zero).^M
** ^M
**   PR_GetEnv(), PR_SetEnv() provide a consistent behavior ^M
**   across all supported platforms. There are, however, some^M
**   restrictions and some practices you must use to achieve^M
**   consistent results everywhere.^M
** ^M
**   When manipulating the environment there is no way to un-set^M
**   an environment variable across all platforms. We suggest^M
**   you interpret the return of a pointer to null-string to^M
**   mean the same as a return of NULL from PR_GetEnv().^M
** ^M
**   A call to PR_SetEnv() where the parameter is of the form^M
**   "name" will return PR_FAILURE; the environment remains^M
**   unchanged. A call to PR_SetEnv() where the parameter is^M
**   of the form "name=" may un-set the envrionment variable on^M
**   some platforms; on others it may set the value of the^M
**   environment variable to the null-string.^M
** ^M
**   For example, to test for NULL return or return of the^M
**   null-string from PR_GetEnv(), use the following code^M
**   fragment:^M
** ^M
**      char *val = PR_GetEnv("foo");^M
**      if ((NULL == val) || ('\0' == *val)) { ^M
**          ... interpret this as un-set ... ^M
**      }^M
** ^M
**   The caller must ensure that the string passed^M
**   to PR_SetEnv() is persistent. That is: The string should^M
**   not be on the stack, where it can be overwritten^M
**   on return from the function calling PR_SetEnv().^M
**   Similarly, the string passed to PR_SetEnv() must not be^M
**   overwritten by other actions of the process. ... Some^M
**   platforms use the string by reference rather than copying^M
**   it into the environment space. ... You have been warned!^M
** ^M
**   Use of platform-native functions that manipulate the^M
**   environment (getenv(), putenv(), ^M
**   SetEnvironmentVariable(), etc.) must not be used with^M
**   NSPR's similar functions. The platform-native functions^M
**   may not be thread safe and/or may operate on different^M
**   conceptual environment space than that operated upon by^M
**   NSPR's functions or other environment manipulating^M
**   functions on the same platform. (!)^M
** ^M
*/^M
NSPR_API(char*) PR_GetEnv(const char *var);^M
^M
/*^M
** PR_SetEnv() -- set, unset or change an environment variable^M
** ^M
** Description:^M
** PR_SetEnv() is modeled on the Unix putenv() function.^M
** ^M
** Inputs: ^M
**   string -- pointer to a caller supplied^M
**   constant, persistent string of the form name=value. Where^M
**   name is the name of the environment variable to be set or^M
**   changed; value is the value assigned to the variable.^M
**^M
** Returns: ^M
**   PRStatus.^M
** ^M
** Restrictions: ^M
**   See the Restrictions documented in the description of^M
**   PR_GetEnv() in this header file.^M
** ^M
** ^M
*/^M
NSPR_API(PRStatus) PR_SetEnv(const char *string);^M
^M
/*^M
** DEPRECATED.  Use PR_SetEnv() instead.^M
*/^M
#ifdef XP_MAC^M
NSPR_API(PRIntn) PR_PutEnv(const char *string);^M
#endif^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prenv_h___ */^M
