/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
** File:		prinrval.h^M
** Description:	API to interval timing functions of NSPR.^M
**^M
**^M
** NSPR provides interval times that are independent of network time^M
** of day values. Interval times are (in theory) accurate regardless^M
** of host processing requirements and also very cheap to acquire. It^M
** is expected that getting an interval time while in a synchronized^M
** function (holding one's lock).^M
**/^M
^M
#if !defined(prinrval_h)^M
#define prinrval_h^M
^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/**********************************************************************/^M
/************************* TYPES AND CONSTANTS ************************/^M
/**********************************************************************/^M
^M
typedef PRUint32 PRIntervalTime;^M
^M
/***********************************************************************^M
** DEFINES:     PR_INTERVAL_MIN^M
**              PR_INTERVAL_MAX^M
** DESCRIPTION:^M
**  These two constants define the range (in ticks / second) of the^M
**  platform dependent type, PRIntervalTime. These constants bound both^M
**  the period and the resolution of a PRIntervalTime. ^M
***********************************************************************/^M
#define PR_INTERVAL_MIN 1000UL^M
#define PR_INTERVAL_MAX 100000UL^M
^M
/***********************************************************************^M
** DEFINES:     PR_INTERVAL_NO_WAIT^M
**              PR_INTERVAL_NO_TIMEOUT^M
** DESCRIPTION:^M
**  Two reserved constants are defined in the PRIntervalTime namespace.^M
**  They are used to indicate that the process should wait no time (return^M
**  immediately) or wait forever (never time out), respectively.^M
***********************************************************************/^M
#define PR_INTERVAL_NO_WAIT 0UL^M
#define PR_INTERVAL_NO_TIMEOUT 0xffffffffUL^M
^M
/**********************************************************************/^M
/****************************** FUNCTIONS *****************************/^M
/**********************************************************************/^M
^M
/***********************************************************************^M
** FUNCTION:    PR_IntervalNow^M
** DESCRIPTION:^M
**  Return the value of NSPR's free running interval timer. That timer^M
**  can be used to establish epochs and determine intervals (be computing^M
**  the difference between two times).^M
** INPUTS:      void^M
** OUTPUTS:     void^M
** RETURN:      PRIntervalTime^M
**  ^M
** SIDE EFFECTS:^M
**  None^M
** RESTRICTIONS:^M
**  The units of PRIntervalTime are platform dependent. They are chosen^M
**  such that they are appropriate for the host OS, yet provide sufficient^M
**  resolution and period to be useful to clients. ^M
** MEMORY:      N/A^M
** ALGORITHM:   Platform dependent^M
***********************************************************************/^M
NSPR_API(PRIntervalTime) PR_IntervalNow(void);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_TicksPerSecond^M
** DESCRIPTION:^M
**  Return the number of ticks per second for PR_IntervalNow's clock.^M
**  The value will be in the range [PR_INTERVAL_MIN..PR_INTERVAL_MAX].^M
** INPUTS:      void^M
** OUTPUTS:     void^M
** RETURN:      PRUint32^M
**  ^M
** SIDE EFFECTS:^M
**  None^M
** RESTRICTIONS:^M
**  None^M
** MEMORY:      N/A^M
** ALGORITHM:   N/A^M
***********************************************************************/^M
NSPR_API(PRUint32) PR_TicksPerSecond(void);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_SecondsToInterval^M
**              PR_MillisecondsToInterval^M
**              PR_MicrosecondsToInterval^M
** DESCRIPTION:^M
**  Convert standard clock units to platform dependent intervals.^M
** INPUTS:      PRUint32^M
** OUTPUTS:     void^M
** RETURN:      PRIntervalTime^M
**  ^M
** SIDE EFFECTS:^M
**  None^M
** RESTRICTIONS:^M
**  Conversion may cause overflow, which is not reported.^M
** MEMORY:      N/A^M
** ALGORITHM:   N/A^M
***********************************************************************/^M
NSPR_API(PRIntervalTime) PR_SecondsToInterval(PRUint32 seconds);^M
NSPR_API(PRIntervalTime) PR_MillisecondsToInterval(PRUint32 milli);^M
NSPR_API(PRIntervalTime) PR_MicrosecondsToInterval(PRUint32 micro);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_IntervalToSeconds^M
**              PR_IntervalToMilliseconds^M
**              PR_IntervalToMicroseconds^M
** DESCRIPTION:^M
**  Convert platform dependent intervals to standard clock units.^M
** INPUTS:      PRIntervalTime^M
** OUTPUTS:     void^M
** RETURN:      PRUint32^M
**  ^M
** SIDE EFFECTS:^M
**  None^M
** RESTRICTIONS:^M
**  Conversion may cause overflow, which is not reported.^M
** MEMORY:      N/A^M
** ALGORITHM:   N/A^M
***********************************************************************/^M
NSPR_API(PRUint32) PR_IntervalToSeconds(PRIntervalTime ticks);^M
NSPR_API(PRUint32) PR_IntervalToMilliseconds(PRIntervalTime ticks);^M
NSPR_API(PRUint32) PR_IntervalToMicroseconds(PRIntervalTime ticks);^M
^M
PR_END_EXTERN_C^M
^M
^M
#endif /* !defined(prinrval_h) */^M
^M
/* prinrval.h */^M
