/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1999-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
 * File: pripcsem.h^M
 *^M
 * Description: named semaphores for interprocess^M
 * synchronization^M
 *^M
 * Unrelated processes obtain access to a shared semaphore^M
 * by specifying its name.^M
 *^M
 * Our goal is to support named semaphores on at least^M
 * Unix and Win32 platforms.  The implementation will use^M
 * one of the three native semaphore APIs: POSIX, System V,^M
 * and Win32.^M
 *^M
 * Because POSIX named semaphores have kernel persistence,^M
 * we are forced to have a delete function in this API.^M
 */^M
^M
#ifndef pripcsem_h___^M
#define pripcsem_h___^M
^M
#include "prtypes.h"^M
#include "prio.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
 * PRSem is an opaque structure that represents a named^M
 * semaphore.^M
 */^M
typedef struct PRSem PRSem;^M
^M
/*^M
 * PR_OpenSemaphore --^M
 *^M
 * Create or open a named semaphore with the specified name.^M
 * A handle to the semaphore is returned.^M
 *^M
 * If the named semaphore doesn't exist and the PR_SEM_CREATE^M
 * flag is specified, the named semaphore is created.  The^M
 * created semaphore needs to be removed from the system with^M
 * a PR_DeleteSemaphore call.^M
 *^M
 * If PR_SEM_CREATE is specified, the third argument is the^M
 * access permission bits of the new semaphore (same^M
 * interpretation as the mode argument to PR_Open) and the^M
 * fourth argument is the initial value of the new semaphore.^M
 * If PR_SEM_CREATE is not specified, the third and fourth^M
 * arguments are ignored.^M
 */^M
^M
#define PR_SEM_CREATE 0x1  /* create if not exist */^M
#define PR_SEM_EXCL   0x2  /* fail if already exists */^M
^M
NSPR_API(PRSem *) PR_OpenSemaphore(^M
    const char *name, PRIntn flags, PRIntn mode, PRUintn value);^M
^M
/*^M
 * PR_WaitSemaphore --^M
 *^M
 * If the value of the semaphore is > 0, decrement the value and return.^M
 * If the value is 0, sleep until the value becomes > 0, then decrement^M
 * the value and return.^M
 *^M
 * The "test and decrement" operation is performed atomically.^M
 */^M
^M
NSPR_API(PRStatus) PR_WaitSemaphore(PRSem *sem);^M
^M
/*^M
 * PR_PostSemaphore --^M
 *^M
 * Increment the value of the named semaphore by 1.^M
 */^M
^M
NSPR_API(PRStatus) PR_PostSemaphore(PRSem *sem);^M
^M
/*^M
 * PR_CloseSemaphore --^M
 *^M
 * Close a named semaphore handle.^M
 */^M
^M
NSPR_API(PRStatus) PR_CloseSemaphore(PRSem *sem);^M
^M
/*^M
 * PR_DeleteSemaphore --^M
 *^M
 * Remove a named semaphore from the system.^M
 */^M
^M
NSPR_API(PRStatus) PR_DeleteSemaphore(const char *name);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* pripcsem_h___ */^M
