/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
** File:	pprio.h^M
**^M
** Description:	Private definitions for I/O related structures^M
*/^M
^M
#ifndef pprio_h___^M
#define pprio_h___^M
^M
#include "prtypes.h"^M
#include "prio.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/************************************************************************/^M
/************************************************************************/^M
^M
/* Return the method tables for files, tcp sockets and udp sockets */^M
NSPR_API(const PRIOMethods*)    PR_GetFileMethods(void);^M
NSPR_API(const PRIOMethods*)    PR_GetTCPMethods(void);^M
NSPR_API(const PRIOMethods*)    PR_GetUDPMethods(void);^M
NSPR_API(const PRIOMethods*)    PR_GetPipeMethods(void);^M
^M
/*^M
** Convert a NSPR Socket Handle to a Native Socket handle.^M
** This function will be obsoleted with the next release; avoid using it.^M
*/^M
NSPR_API(PRInt32)      PR_FileDesc2NativeHandle(PRFileDesc *);^M
NSPR_API(void)         PR_ChangeFileDescNativeHandle(PRFileDesc *, PRInt32);^M
NSPR_API(PRFileDesc*)  PR_AllocFileDesc(PRInt32 osfd,^M
                                         const PRIOMethods *methods);^M
NSPR_API(void)         PR_FreeFileDesc(PRFileDesc *fd);^M
/*^M
** Import an existing OS file to NSPR. ^M
*/^M
NSPR_API(PRFileDesc*)  PR_ImportFile(PRInt32 osfd);^M
NSPR_API(PRFileDesc*)  PR_ImportPipe(PRInt32 osfd);^M
NSPR_API(PRFileDesc*)  PR_ImportTCPSocket(PRInt32 osfd);^M
NSPR_API(PRFileDesc*)  PR_ImportUDPSocket(PRInt32 osfd);^M
^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_CreateSocketPollFd^M
 * DESCRIPTION:^M
 *     Create a PRFileDesc wrapper for a native socket handle, for use with^M
 *	   PR_Poll only^M
 * INPUTS:^M
 *     None^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRFileDesc*^M
 *     Upon successful completion, PR_CreateSocketPollFd returns a pointer^M
 *     to the PRFileDesc created for the native socket handle^M
 *     Returns a NULL pointer if the create of a new PRFileDesc failed^M
 *^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRFileDesc*)	PR_CreateSocketPollFd(PRInt32 osfd);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_DestroySocketPollFd^M
 * DESCRIPTION:^M
 *     Destroy the PRFileDesc wrapper created by PR_CreateSocketPollFd^M
 * INPUTS:^M
 *     None^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRFileDesc*^M
 *     Upon successful completion, PR_DestroySocketPollFd returns^M
 *	   PR_SUCCESS, else PR_FAILURE^M
 *^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_DestroySocketPollFd(PRFileDesc *fd);^M
^M
^M
/*^M
** Macros for PR_Socket^M
**^M
** Socket types: PR_SOCK_STREAM, PR_SOCK_DGRAM^M
*/^M
^M
#ifdef WIN32^M
^M
#define PR_SOCK_STREAM 1^M
#define PR_SOCK_DGRAM 2^M
^M
#else /* WIN32 */^M
^M
#define PR_SOCK_STREAM SOCK_STREAM^M
#define PR_SOCK_DGRAM SOCK_DGRAM^M
^M
#endif /* WIN32 */^M
^M
/*^M
** Create a new Socket; this function is obsolete.^M
*/^M
NSPR_API(PRFileDesc*)	PR_Socket(PRInt32 domain, PRInt32 type, PRInt32 proto);^M
^M
/* FUNCTION: PR_LockFile^M
** DESCRIPTION:^M
**    Lock a file for exclusive access.^M
** RETURNS:^M
**    PR_SUCCESS when the lock is held^M
**    PR_FAILURE otherwise^M
*/^M
NSPR_API(PRStatus) PR_LockFile(PRFileDesc *fd);^M
^M
/* FUNCTION: PR_TLockFile^M
** DESCRIPTION:^M
**    Test and Lock a file for exclusive access.  Do not block if the^M
**    file cannot be locked immediately.^M
** RETURNS:^M
**    PR_SUCCESS when the lock is held^M
**    PR_FAILURE otherwise^M
*/^M
NSPR_API(PRStatus) PR_TLockFile(PRFileDesc *fd);^M
^M
/* FUNCTION: PR_UnlockFile^M
** DESCRIPTION:^M
**    Unlock a file which has been previously locked successfully by this^M
**    process.^M
** RETURNS:^M
**    PR_SUCCESS when the lock is released^M
**    PR_FAILURE otherwise^M
*/^M
NSPR_API(PRStatus) PR_UnlockFile(PRFileDesc *fd);^M
^M
/*^M
** Emulate acceptread by accept and recv.^M
*/^M
NSPR_API(PRInt32) PR_EmulateAcceptRead(PRFileDesc *sd, PRFileDesc **nd,^M
    PRNetAddr **raddr, void *buf, PRInt32 amount, PRIntervalTime timeout);^M
^M
/*^M
** Emulate sendfile by reading from the file and writing to the socket.^M
** The file is memory-mapped if memory-mapped files are supported.^M
*/^M
NSPR_API(PRInt32) PR_EmulateSendFile(^M
    PRFileDesc *networkSocket, PRSendFileData *sendData,^M
    PRTransmitFileFlags flags, PRIntervalTime timeout);^M
^M
#ifdef WIN32^M
/* FUNCTION: PR_NTFast_AcceptRead^M
** DESCRIPTION:^M
**    NT has the notion of an "accept context", which is only needed in^M
**    order to make certain calls.  By default, a socket connected via^M
**    AcceptEx can only do a limited number of things without updating^M
**    the acceptcontext.  The generic version of PR_AcceptRead always^M
**    updates the accept context.  This version does not.^M
**/^M
NSPR_API(PRInt32) PR_NTFast_AcceptRead(PRFileDesc *sd, PRFileDesc **nd,^M
              PRNetAddr **raddr, void *buf, PRInt32 amount, PRIntervalTime t);^M
^M
typedef void (*_PR_AcceptTimeoutCallback)(void *);^M
^M
/* FUNCTION: PR_NTFast_AcceptRead_WithTimeoutCallback^M
** DESCRIPTION:^M
**    The AcceptEx call combines the accept with the read function.  However,^M
**    our daemon threads need to be able to wakeup and reliably flush their^M
**    log buffers if the Accept times out.  However, with the current blocking^M
**    interface to AcceptRead, there is no way for us to timeout the Accept;^M
**    this is because when we timeout the Read, we can close the newly ^M
**    socket and continue; but when we timeout the accept itself, there is no^M
**    new socket to timeout.  So instead, this version of the function is^M
**    provided.  After the initial timeout period elapses on the accept()^M
**    portion of the function, it will call the callback routine and then^M
**    continue the accept.   If the timeout occurs on the read, it will ^M
**    close the connection and return error.^M
*/^M
NSPR_API(PRInt32) PR_NTFast_AcceptRead_WithTimeoutCallback(^M
              PRFileDesc *sd, ^M
              PRFileDesc **nd,^M
              PRNetAddr **raddr, ^M
              void *buf, ^M
              PRInt32 amount, ^M
              PRIntervalTime t,^M
              _PR_AcceptTimeoutCallback callback, ^M
              void *callback_arg);^M
^M
/* FUNCTION: PR_NTFast_Accept^M
** DESCRIPTION:^M
**    NT has the notion of an "accept context", which is only needed in^M
**    order to make certain calls.  By default, a socket connected via^M
**    AcceptEx can only do a limited number of things without updating^M
**    the acceptcontext.  The generic version of PR_Accept always^M
**    updates the accept context.  This version does not.^M
**/^M
NSPR_API(PRFileDesc*)	PR_NTFast_Accept(PRFileDesc *fd, PRNetAddr *addr,^M
                                                PRIntervalTime timeout);^M
^M
/* FUNCTION: PR_NTFast_Update^M
** DESCRIPTION:^M
**    For sockets accepted with PR_NTFast_Accept or PR_NTFastAcceptRead,^M
**    this function will update the accept context for those sockets,^M
**    so that the socket can make general purpose socket calls.^M
**    Without calling this, the only operations supported on the socket^M
**    Are PR_Read, PR_Write, PR_Transmitfile, and PR_Close.^M
*/^M
NSPR_API(void) PR_NTFast_UpdateAcceptContext(PRFileDesc *acceptSock, ^M
                                        PRFileDesc *listenSock);^M
^M
^M
/* FUNCTION: PR_NT_CancelIo^M
** DESCRIPTION:^M
**    Cancel IO operations on fd.^M
*/^M
NSPR_API(PRStatus) PR_NT_CancelIo(PRFileDesc *fd);^M
^M
^M
#endif /* WIN32 */^M
^M
/*^M
** Need external access to this on Mac so we can first set up our faux^M
** environment vars^M
*/^M
#ifdef XP_MAC^M
NSPR_API(void) PR_Init_Log(void);^M
#endif^M
^M
^M
PR_END_EXTERN_C^M
^M
#endif /* pprio_h___ */^M
