/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef pprthred_h___^M
#define pprthred_h___^M
^M
/*^M
** API for PR private functions.  These calls are to be used by internal^M
** developers only.^M
*/^M
#include "nspr.h"^M
^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*---------------------------------------------------------------------------^M
** THREAD PRIVATE FUNCTIONS^M
---------------------------------------------------------------------------*/^M
^M
/*^M
** Associate a thread object with an existing native thread.^M
** 	"type" is the type of thread object to attach^M
** 	"priority" is the priority to assign to the thread^M
** 	"stack" defines the shape of the threads stack^M
**^M
** This can return NULL if some kind of error occurs, or if memory is^M
** tight. This call invokes "start(obj,arg)" and returns when the^M
** function returns. The thread object is automatically destroyed.^M
**^M
** This call is not normally needed unless you create your own native^M
** thread. PR_Init does this automatically for the primordial thread.^M
*/^M
NSPR_API(PRThread*) PR_AttachThread(PRThreadType type,^M
                                     PRThreadPriority priority,^M
				     PRThreadStack *stack);^M
^M
/*^M
** Detach the nspr thread from the currently executing native thread.^M
** The thread object will be destroyed and all related data attached^M
** to it. The exit procs will be invoked.^M
**^M
** This call is not normally needed unless you create your own native^M
** thread. PR_Exit will automatially detach the nspr thread object^M
** created by PR_Init for the primordial thread.^M
**^M
** This call returns after the nspr thread object is destroyed.^M
*/^M
NSPR_API(void) PR_DetachThread(void);^M
^M
/*^M
** Get the id of the named thread. Each thread is assigned a unique id^M
** when it is created or attached.^M
*/^M
NSPR_API(PRUint32) PR_GetThreadID(PRThread *thread);^M
^M
/*^M
** Set the procedure that is called when a thread is dumped. The procedure^M
** will be applied to the argument, arg, when called. Setting the procedure^M
** to NULL effectively removes it.^M
*/^M
typedef void (*PRThreadDumpProc)(PRFileDesc *fd, PRThread *t, void *arg);^M
NSPR_API(void) PR_SetThreadDumpProc(^M
    PRThread* thread, PRThreadDumpProc dump, void *arg);^M
^M
/*^M
** Get this thread's affinity mask.  The affinity mask is a 32 bit quantity^M
** marking a bit for each processor this process is allowed to run on.^M
** The processor mask is returned in the mask argument.^M
** The least-significant-bit represents processor 0.^M
**^M
** Returns 0 on success, -1 on failure.^M
*/^M
NSPR_API(PRInt32) PR_GetThreadAffinityMask(PRThread *thread, PRUint32 *mask);^M
^M
/*^M
** Set this thread's affinity mask.  ^M
**^M
** Returns 0 on success, -1 on failure.^M
*/^M
NSPR_API(PRInt32) PR_SetThreadAffinityMask(PRThread *thread, PRUint32 mask );^M
^M
/*^M
** Set the default CPU Affinity mask.^M
**^M
*/^M
NSPR_API(PRInt32) PR_SetCPUAffinityMask(PRUint32 mask);^M
^M
/*^M
** Show status of all threads to standard error output.^M
*/^M
NSPR_API(void) PR_ShowStatus(void);^M
^M
/*^M
** Set thread recycle mode to on (1) or off (0)^M
*/^M
NSPR_API(void) PR_SetThreadRecycleMode(PRUint32 flag);^M
^M
^M
/*---------------------------------------------------------------------------^M
** THREAD PRIVATE FUNCTIONS FOR GARBAGE COLLECTIBLE THREADS           ^M
---------------------------------------------------------------------------*/^M
^M
/* ^M
** Only Garbage collectible threads participate in resume all, suspend all and ^M
** enumeration operations.  They are also different during creation when^M
** platform specific action may be needed (For example, all Solaris GC able^M
** threads are bound threads).^M
*/^M
^M
/*^M
** Same as PR_CreateThread except that the thread is marked as garbage^M
** collectible.^M
*/^M
NSPR_API(PRThread*) PR_CreateThreadGCAble(PRThreadType type,^M
				     void (*start)(void *arg),^M
				     void *arg,^M
				     PRThreadPriority priority,^M
				     PRThreadScope scope,^M
				     PRThreadState state,^M
				     PRUint32 stackSize);^M
^M
/*^M
** Same as PR_AttachThread except that the thread being attached is marked as ^M
** garbage collectible.^M
*/^M
NSPR_API(PRThread*) PR_AttachThreadGCAble(PRThreadType type,^M
					PRThreadPriority priority,^M
					PRThreadStack *stack);^M
^M
/*^M
** Mark the thread as garbage collectible.^M
*/^M
NSPR_API(void) PR_SetThreadGCAble(void);^M
^M
/*^M
** Unmark the thread as garbage collectible.^M
*/^M
NSPR_API(void) PR_ClearThreadGCAble(void);^M
^M
/*^M
** This routine prevents all other GC able threads from running. This call is needed by ^M
** the garbage collector.^M
*/^M
NSPR_API(void) PR_SuspendAll(void);^M
^M
/*^M
** This routine unblocks all other GC able threads that were suspended from running by ^M
** PR_SuspendAll(). This call is needed by the garbage collector.^M
*/^M
NSPR_API(void) PR_ResumeAll(void);^M
^M
/*^M
** Return the thread stack pointer of the given thread. ^M
** Needed by the garbage collector.^M
*/^M
NSPR_API(void *) PR_GetSP(PRThread *thread);^M
^M
/*^M
** Save the registers that the GC would find interesting into the thread^M
** "t". isCurrent will be non-zero if the thread state that is being^M
** saved is the currently executing thread. Return the address of the^M
** first register to be scanned as well as the number of registers to^M
** scan in "np".^M
**^M
** If "isCurrent" is non-zero then it is allowed for the thread context^M
** area to be used as scratch storage to hold just the registers^M
** necessary for scanning.^M
**^M
** This function simply calls the internal function _MD_HomeGCRegisters().^M
*/^M
NSPR_API(PRWord *) PR_GetGCRegisters(PRThread *t, int isCurrent, int *np);^M
^M
/*^M
** (Get|Set)ExecutionEnvironent^M
**^M
** Used by Java to associate it's execution environment so garbage collector^M
** can find it. If return is NULL, then it's probably not a collectable thread.^M
**^M
** There's no locking required around these calls.^M
*/^M
NSPR_API(void*) GetExecutionEnvironment(PRThread *thread);^M
NSPR_API(void) SetExecutionEnvironment(PRThread* thread, void *environment);^M
^M
/*^M
** Enumeration function that applies "func(thread,i,arg)" to each active^M
** thread in the process. The enumerator returns PR_SUCCESS if the enumeration^M
** should continue, any other value is considered failure, and enumeration^M
** stops, returning the failure value from PR_EnumerateThreads.^M
** Needed by the garbage collector.^M
*/^M
typedef PRStatus (PR_CALLBACK *PREnumerator)(PRThread *t, int i, void *arg);^M
NSPR_API(PRStatus) PR_EnumerateThreads(PREnumerator func, void *arg);^M
^M
/* ^M
** Signature of a thread stack scanning function. It is applied to every^M
** contiguous group of potential pointers within a thread. Count denotes the^M
** number of pointers. ^M
*/^M
typedef PRStatus ^M
(PR_CALLBACK *PRScanStackFun)(PRThread* t,^M
			      void** baseAddr, PRUword count, void* closure);^M
^M
/*^M
** Applies scanFun to all contiguous groups of potential pointers ^M
** within a thread. This includes the stack, registers, and thread-local^M
** data. If scanFun returns a status value other than PR_SUCCESS the scan^M
** is aborted, and the status value is returned. ^M
*/^M
NSPR_API(PRStatus)^M
PR_ThreadScanStackPointers(PRThread* t,^M
                           PRScanStackFun scanFun, void* scanClosure);^M
^M
/* ^M
** Calls PR_ThreadScanStackPointers for every thread.^M
*/^M
NSPR_API(PRStatus)^M
PR_ScanStackPointers(PRScanStackFun scanFun, void* scanClosure);^M
^M
/*^M
** Returns a conservative estimate on the amount of stack space left^M
** on a thread in bytes, sufficient for making decisions about whether ^M
** to continue recursing or not.^M
*/^M
NSPR_API(PRUword)^M
PR_GetStackSpaceLeft(PRThread* t);^M
^M
/*---------------------------------------------------------------------------^M
** THREAD CPU PRIVATE FUNCTIONS             ^M
---------------------------------------------------------------------------*/^M
^M
/*^M
** Get a pointer to the primordial CPU.^M
*/^M
NSPR_API(struct _PRCPU *) _PR_GetPrimordialCPU(void);^M
^M
/*---------------------------------------------------------------------------^M
** THREAD SYNCHRONIZATION PRIVATE FUNCTIONS^M
---------------------------------------------------------------------------*/^M
^M
/*^M
** Create a new named monitor (named for debugging purposes).^M
**  Monitors are re-entrant locks with a built-in condition variable.^M
**^M
** This may fail if memory is tight or if some operating system resource^M
** is low.^M
*/^M
NSPR_API(PRMonitor*) PR_NewNamedMonitor(const char* name);^M
^M
/*^M
** Test and then lock the lock if it's not already locked by some other^M
** thread. Return PR_FALSE if some other thread owned the lock at the^M
** time of the call.^M
*/^M
NSPR_API(PRBool) PR_TestAndLock(PRLock *lock);^M
^M
/*^M
** Test and then enter the mutex associated with the monitor if it's not^M
** already entered by some other thread. Return PR_FALSE if some other^M
** thread owned the mutex at the time of the call.^M
*/^M
NSPR_API(PRBool) PR_TestAndEnterMonitor(PRMonitor *mon);^M
^M
/*^M
** Return the number of times that the current thread has entered the^M
** mutex. Returns zero if the current thread has not entered the mutex.^M
*/^M
NSPR_API(PRIntn) PR_GetMonitorEntryCount(PRMonitor *mon);^M
^M
/*^M
** Just like PR_CEnterMonitor except that if the monitor is owned by^M
** another thread NULL is returned.^M
*/^M
NSPR_API(PRMonitor*) PR_CTestAndEnterMonitor(void *address);^M
^M
/*---------------------------------------------------------------------------^M
** PLATFORM-SPECIFIC THREAD SYNCHRONIZATION FUNCTIONS^M
---------------------------------------------------------------------------*/^M
#if defined(XP_MAC)^M
^M
NSPR_API(void) PR_Mac_WaitForAsyncNotify(PRIntervalTime timeout);^M
NSPR_API(void) PR_Mac_PostAsyncNotify(PRThread *thread);^M
^M
#endif /* XP_MAC */^M
^M
/*---------------------------------------------------------------------------^M
** PLATFORM-SPECIFIC INITIALIZATION FUNCTIONS^M
---------------------------------------------------------------------------*/^M
#if defined(IRIX)^M
/*^M
** Irix specific initialization funtion to be called before PR_Init^M
** is called by the application. Sets the CONF_INITUSERS and CONF_INITSIZE^M
** attributes of the shared arena set up by nspr.^M
**^M
** The environment variables _NSPR_IRIX_INITUSERS and _NSPR_IRIX_INITSIZE^M
** can also be used to set these arena attributes. If _NSPR_IRIX_INITUSERS^M
** is set, but not _NSPR_IRIX_INITSIZE, the value of the CONF_INITSIZE^M
** attribute of the nspr arena is scaled as a function of the^M
** _NSPR_IRIX_INITUSERS value.^M
** ^M
** If the _PR_Irix_Set_Arena_Params() is called in addition to setting the^M
** environment variables, the values of the environment variables are used.^M
** ^M
*/^M
NSPR_API(void) _PR_Irix_Set_Arena_Params(PRInt32 initusers, PRInt32 initsize);^M
^M
#endif /* IRIX */^M
^M
/* I think PR_GetMonitorEntryCount is useless. All you really want is this... */^M
#define PR_InMonitor(m)		(PR_GetMonitorEntryCount(m) > 0)^M
^M
/*---------------------------------------------------------------------------^M
** Special X-Lock hack for client^M
---------------------------------------------------------------------------*/^M
^M
#ifdef XP_UNIX^M
extern void PR_XLock(void);^M
extern void PR_XUnlock(void);^M
extern PRBool PR_XIsLocked(void);^M
#endif /* XP_UNIX */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* pprthred_h___ */^M
