/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prlink_h___^M
#define prlink_h___^M
^M
/*^M
** API to static and dynamic linking.^M
*/^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
typedef struct PRLibrary PRLibrary;^M
^M
typedef struct PRStaticLinkTable {^M
    const char *name;^M
    void (*fp)();^M
} PRStaticLinkTable;^M
^M
/*^M
** Change the default library path to the given string. The string is^M
** copied. This call will fail if it runs out of memory.^M
**^M
** The string provided as 'path' is copied. The caller can do whatever is^M
** convenient with the argument when the function is complete.^M
*/^M
NSPR_API(PRStatus) PR_SetLibraryPath(const char *path);^M
^M
/*^M
** Return a character string which contains the path used to search for^M
** dynamically loadable libraries.^M
**^M
** The returned value is basically a copy of a PR_SetLibraryPath().^M
** The storage is allocated by the runtime and becomes the responsibilty^M
** of the caller.^M
*/^M
NSPR_API(char*) PR_GetLibraryPath(void);^M
^M
/*^M
** Given a directory name "dir" and a library name "lib" construct a full^M
** path name that will refer to the actual dynamically loaded^M
** library. This does not test for existance of said file, it just^M
** constructs the full filename. The name constructed is system dependent^M
** and prepared for PR_LoadLibrary. The result must be free'd when the^M
** caller is done with it.^M
**^M
** The storage for the result is allocated by the runtime and becomes the^M
** responsibility of the caller.^M
*/^M
NSPR_API(char*) PR_GetLibraryName(const char *dir, const char *lib);^M
^M
/*^M
**^M
** Free the memory allocated, for the caller, by PR_GetLibraryName^M
*/^M
NSPR_API(void) PR_FreeLibraryName(char *mem);^M
^M
/*^M
** Given a library "name" try to load the library. The argument "name"^M
** is a machine-dependent name for the library, such as the full pathname^M
** returned by PR_GetLibraryName.  If the library is already loaded,^M
** this function will avoid loading the library twice.^M
**^M
** If the library is loaded successfully, then a pointer to the PRLibrary^M
** structure representing the library is returned.  Otherwise, NULL is^M
** returned.^M
**^M
** This increments the reference count of the library.^M
*/^M
NSPR_API(PRLibrary*) PR_LoadLibrary(const char *name);^M
^M
/*^M
** Each operating system has its preferred way of specifying^M
** a file in the file system.  Most operating systems use^M
** a pathname.  Mac OS, on the other hand, uses the FSSpec^M
** structure to specify a file. PRLibSpec allows NSPR clients^M
** to use the type of file specification that is most efficient^M
** for a particular platform.^M
**^M
** On some operating systems such as Mac OS, a shared library may^M
** contain code fragments that can be individually loaded.^M
** PRLibSpec also allows NSPR clients to identify a code fragment^M
** in a library, if code fragments are supported by the OS.^M
** A code fragment can be specified by name or by an integer index.^M
**^M
** Right now PRLibSpec supports three types of library specification:^M
** a pathname, a Mac code fragment by name, and a Mac code fragment^M
** by index.^M
*/^M
^M
typedef enum PRLibSpecType {^M
    PR_LibSpec_Pathname,^M
    PR_LibSpec_MacNamedFragment,^M
    PR_LibSpec_MacIndexedFragment^M
} PRLibSpecType;^M
^M
struct FSSpec; /* Mac OS FSSpec */^M
^M
typedef struct PRLibSpec {^M
    PRLibSpecType type;^M
    union {^M
        /* if type is PR_LibSpec_Pathname */^M
        const char *pathname;^M
^M
        /* if type is PR_LibSpec_MacNamedFragment */^M
        struct {^M
            const struct FSSpec *fsspec;^M
            const char *name;^M
        } mac_named_fragment;^M
^M
        /* if type is PR_LibSpec_MacIndexedFragment */^M
        struct {^M
            const struct FSSpec *fsspec;^M
            PRUint32 index;^M
        } mac_indexed_fragment;^M
    } value;^M
} PRLibSpec;^M
^M
/*^M
** The following bit flags may be or'd together and passed^M
** as the 'flags' argument to PR_LoadLibraryWithFlags.^M
** Flags not supported by the underlying OS are ignored.^M
*/^M
^M
#define PR_LD_LAZY   0x1  /* equivalent to RTLD_LAZY on Unix */^M
#define PR_LD_NOW    0x2  /* equivalent to RTLD_NOW on Unix */^M
#define PR_LD_GLOBAL 0x4  /* equivalent to RTLD_GLOBAL on Unix */^M
#define PR_LD_LOCAL  0x8  /* equivalent to RTLD_LOCAL on Unix */^M
^M
/*^M
** Load the specified library, in the manner specified by 'flags'.^M
*/^M
^M
NSPR_API(PRLibrary *)^M
PR_LoadLibraryWithFlags(^M
    PRLibSpec libSpec,    /* the shared library */^M
    PRIntn flags          /* flags that affect the loading */^M
);^M
^M
/*^M
** Unload a previously loaded library. If the library was a static^M
** library then the static link table will no longer be referenced. The^M
** associated PRLibrary object is freed.^M
**^M
** PR_FAILURE is returned if the library cannot be unloaded.^M
**^M
** This function decrements the reference count of the library.^M
*/^M
NSPR_API(PRStatus) PR_UnloadLibrary(PRLibrary *lib);^M
^M
/*^M
** Given the name of a procedure, return the address of the function that^M
** implements the procedure, or NULL if no such function can be^M
** found. This does not find symbols in the main program (the ".exe");^M
** use PR_LoadStaticLibrary to register symbols in the main program.^M
**^M
** This function does not modify the reference count of the library.^M
*/^M
NSPR_API(void*) PR_FindSymbol(PRLibrary *lib, const char *name);^M
^M
/*^M
** Similar to PR_FindSymbol, except that the return value is a pointer to^M
** a function, and not a pointer to void. Casting between a data pointer^M
** and a function pointer is not portable according to the C standard.^M
** Any function pointer can be cast to any other function pointer.^M
**^M
** This function does not modify the reference count of the library.^M
*/^M
typedef void (*PRFuncPtr)();^M
NSPR_API(PRFuncPtr) PR_FindFunctionSymbol(PRLibrary *lib, const char *name);^M
^M
/*^M
** Finds a symbol in one of the currently loaded libraries. Given the^M
** name of a procedure, return the address of the function that^M
** implements the procedure, and return the library that contains that^M
** symbol, or NULL if no such function can be found. This does not find^M
** symbols in the main program (the ".exe"); use PR_AddStaticLibrary to^M
** register symbols in the main program.  ^M
**^M
** This increments the reference count of the library.^M
*/^M
NSPR_API(void*) PR_FindSymbolAndLibrary(const char *name,^M
						      PRLibrary* *lib);^M
^M
/*^M
** Similar to PR_FindSymbolAndLibrary, except that the return value is^M
** a pointer to a function, and not a pointer to void. Casting between a^M
** data pointer and a function pointer is not portable according to the C^M
** standard. Any function pointer can be cast to any other function pointer.^M
**^M
** This increments the reference count of the library.^M
*/^M
NSPR_API(PRFuncPtr) PR_FindFunctionSymbolAndLibrary(const char *name,^M
						      PRLibrary* *lib);^M
^M
/*^M
** Register a static link table with the runtime under the name^M
** "name". The symbols present in the static link table will be made^M
** available to PR_FindSymbol. If "name" is null then the symbols will be^M
** made available to the library which represents the executable. The^M
** tables are not copied.^M
**^M
** Returns the library object if successful, null otherwise.^M
**^M
** This increments the reference count of the library.^M
*/^M
NSPR_API(PRLibrary*) PR_LoadStaticLibrary(^M
    const char *name, const PRStaticLinkTable *table);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prlink_h___ */^M
