/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prlog_h___^M
#define prlog_h___^M
^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** prlog.h -- Declare interfaces to NSPR's Logging service^M
**^M
** NSPR provides a logging service that is used by NSPR itself and is^M
** available to client programs.^M
**^M
** To use the service from a client program, you should create a^M
** PRLogModuleInfo structure by calling PR_NewLogModule(). After^M
** creating the LogModule, you can write to the log using the PR_LOG()^M
** macro.^M
**^M
** Initialization of the log service is handled by NSPR initialization.^M
**^M
** At execution time, you must enable the log service. To enable the^M
** log service, set the environment variable: NSPR_LOG_MODULES^M
** variable.^M
**^M
** NSPR_LOG_MODULES variable has the form:^M
**^M
**     <moduleName>:<value>[, <moduleName>:<value>]*^M
**^M
** Where:^M
**  <moduleName> is the name passed to PR_NewLogModule().^M
**  <value> is a numeric constant, e.g. 5. This value is the maximum^M
** value of a log event, enumerated by PRLogModuleLevel, that you want^M
** written to the log.^M
** ^M
** For example: to record all events of greater value than or equal to^M
** PR_LOG_ERROR for a LogModule names "gizmo", say:^M
** ^M
** set NSPR_LOG_MODULES=gizmo:2^M
** ^M
** Note that you must specify the numeric value of PR_LOG_ERROR.^M
** ^M
** Special LogModule names are provided for controlling NSPR's log^M
** service at execution time. These controls should be set in the^M
** NSPR_LOG_MODULES environment variable at execution time to affect^M
** NSPR's log service for your application.^M
** ^M
** The special LogModule "all" enables all LogModules. To enable all^M
** LogModule calls to PR_LOG(), say:^M
** ^M
** set NSPR_LOG_MODULES=all:5^M
** ^M
** The special LogModule name "sync" tells the NSPR log service to do^M
** unbuffered logging.^M
** ^M
** The special LogModule name "buffsize:<size>" tells NSPR to set the^M
** log buffer to <size>.^M
**^M
** The environment variable NSPR_LOG_FILE specifies the log file to use^M
** unless the default of "stderr" is acceptable. For MS Windows^M
** systems, NSPR_LOG_FILE can be set to a special value: "WinDebug"^M
** (case sensitive). This value causes PR_LOG() output to be written^M
** using the Windows API OutputDebugString(). OutputDebugString()^M
** writes to the debugger window; some people find this helpful.^M
** ^M
**^M
** To put log messages in your programs, use the PR_LOG macro:^M
**^M
**     PR_LOG(<module>, <level>, (<printfString>, <args>*));^M
**^M
** Where <module> is the address of a PRLogModuleInfo structure, and^M
** <level> is one of the levels defined by the enumeration:^M
** PRLogModuleLevel. <args> is a printf() style of argument list. That^M
** is: (fmtstring, ...).^M
**^M
** Example:^M
** ^M
** main() {^M
**    PRIntn one = 1;^M
**    PRLogModuleInfo * myLm = PR_NewLogModule("gizmo");^M
**    PR_LOG( myLm, PR_LOG_ALWAYS, ("Log this! %d\n", one)); ^M
**    return; ^M
** }^M
** ^M
** Note the use of printf() style arguments as the third agrument(s) to^M
** PR_LOG().^M
** ^M
** After compiling and linking you application, set the environment:^M
** ^M
** set NSPR_LOG_MODULES=gizmo:5^M
** set NSPR_LOG_FILE=logfile.txt^M
** ^M
** When you execute your application, the string "Log this! 1" will be^M
** written to the file "logfile.txt".^M
** ^M
** Note to NSPR engineers: a number of PRLogModuleInfo structures are^M
** defined and initialized in prinit.c. See this module for ideas on^M
** what to log where.^M
** ^M
*/^M
^M
typedef enum PRLogModuleLevel {^M
    PR_LOG_NONE = 0,                /* nothing */^M
    PR_LOG_ALWAYS = 1,              /* always printed */^M
    PR_LOG_ERROR = 2,               /* error messages */^M
    PR_LOG_WARNING = 3,             /* warning messages */^M
    PR_LOG_DEBUG = 4,               /* debug messages */^M
^M
    PR_LOG_NOTICE = PR_LOG_DEBUG,   /* notice messages */^M
    PR_LOG_WARN = PR_LOG_WARNING,   /* warning messages */^M
    PR_LOG_MIN = PR_LOG_DEBUG,      /* minimal debugging messages */^M
    PR_LOG_MAX = PR_LOG_DEBUG       /* maximal debugging messages */^M
} PRLogModuleLevel;^M
^M
/*^M
** One of these structures is created for each module that uses logging.^M
**    "name" is the name of the module^M
**    "level" is the debugging level selected for that module^M
*/^M
typedef struct PRLogModuleInfo {^M
    const char *name;^M
    PRLogModuleLevel level;^M
    struct PRLogModuleInfo *next;^M
} PRLogModuleInfo;^M
^M
/*^M
** Create a new log module.^M
*/^M
NSPR_API(PRLogModuleInfo*) PR_NewLogModule(const char *name);^M
^M
/*^M
** Set the file to use for logging. Returns PR_FALSE if the file cannot^M
** be created^M
*/^M
NSPR_API(PRBool) PR_SetLogFile(const char *name);^M
^M
/*^M
** Set the size of the logging buffer. If "buffer_size" is zero then the^M
** logging becomes "synchronous" (or unbuffered).^M
*/^M
NSPR_API(void) PR_SetLogBuffering(PRIntn buffer_size);^M
^M
/*^M
** Print a string to the log. "fmt" is a PR_snprintf format type. All^M
** messages printed to the log are preceeded by the name of the thread^M
** and a time stamp. Also, the routine provides a missing newline if one^M
** is not provided.^M
*/^M
NSPR_API(void) PR_LogPrint(const char *fmt, ...);^M
^M
/*^M
** Flush the log to its file.^M
*/^M
NSPR_API(void) PR_LogFlush(void);^M
^M
/*^M
** Windoze 16 can't support a large static string space for all of the^M
** various debugging strings so logging is not enabled for it.^M
*/^M
#if (defined(DEBUG) || defined(FORCE_PR_LOG)) && !defined(WIN16)^M
#define PR_LOGGING 1^M
^M
#define PR_LOG_TEST(_module,_level) \^M
    ((_module)->level >= (_level))^M
^M
/*^M
** Log something.^M
**    "module" is the address of a PRLogModuleInfo structure^M
**    "level" is the desired logging level^M
**    "args" is a variable length list of arguments to print, in the following^M
**       format:  ("printf style format string", ...)^M
*/^M
#define PR_LOG(_module,_level,_args)     \^M
    PR_BEGIN_MACRO             \^M
      if (PR_LOG_TEST(_module,_level)) { \^M
      PR_LogPrint _args;         \^M
      }                     \^M
    PR_END_MACRO^M
^M
#else /* (defined(DEBUG) || defined(FORCE_PR_LOG)) && !defined(WIN16) */^M
^M
#undef PR_LOGGING^M
#define PR_LOG_TEST(module,level) 0^M
#define PR_LOG(module,level,args)^M
^M
#endif /* (defined(DEBUG) || defined(FORCE_PR_LOG)) && !defined(WIN16) */^M
^M
#ifndef NO_NSPR_10_SUPPORT^M
^M
#ifdef PR_LOGGING^M
#define PR_LOG_BEGIN    PR_LOG^M
#define PR_LOG_END      PR_LOG^M
#define PR_LOG_DEFINE   PR_NewLogModule^M
#else^M
#define PR_LOG_BEGIN(module,level,args)^M
#define PR_LOG_END(module,level,args)^M
#define PR_LOG_DEFINE(_name)    NULL^M
#endif /* PR_LOGGING */^M
^M
#endif /* NO_NSPR_10_SUPPORT */^M
^M
#if defined(DEBUG)^M
^M
NSPR_API(void) PR_Assert(const char *s, const char *file, PRIntn ln);^M
#define PR_ASSERT(_expr) \^M
    ((_expr)?((void)0):PR_Assert(# _expr,__FILE__,__LINE__))^M
^M
#define PR_NOT_REACHED(_reasonStr) \^M
    PR_Assert(_reasonStr,__FILE__,__LINE__)^M
^M
#else^M
^M
#define PR_ASSERT(expr) ((void) 0)^M
#define PR_NOT_REACHED(reasonStr)^M
^M
#endif /* defined(DEBUG) */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prlog_h___ */^M
