/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prnetdb_h___^M
#define prnetdb_h___^M
^M
#include "prtypes.h"^M
#include "prio.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
^M
/*^M
 *********************************************************************^M
 *  Translate an Internet address to/from a character string^M
 *********************************************************************^M
 */^M
NSPR_API(PRStatus) PR_StringToNetAddr(^M
    const char *string, PRNetAddr *addr);^M
^M
NSPR_API(PRStatus) PR_NetAddrToString(^M
    const PRNetAddr *addr, char *string, PRUint32 size);^M
^M
/*^M
** Structures returned by network data base library.  All addresses are^M
** supplied in host order, and returned in network order (suitable for^M
** use in system calls).^M
*/^M
/*^M
** Beware that WINSOCK.H defines h_addrtype and h_length as short.^M
** Client code does direct struct copies of hostent to PRHostEnt and^M
** hence the ifdef.^M
*/^M
typedef struct PRHostEnt {^M
    char *h_name;       /* official name of host */^M
    char **h_aliases;   /* alias list */^M
#if defined(WIN32) || defined(WIN16)^M
    PRInt16 h_addrtype; /* host address type */^M
    PRInt16 h_length;   /* length of address */^M
#else^M
    PRInt32 h_addrtype; /* host address type */^M
    PRInt32 h_length;   /* length of address */^M
#endif^M
    char **h_addr_list; /* list of addresses from name server */^M
} PRHostEnt;^M
^M
/* A safe size to use that will mostly work... */^M
#if (defined(AIX) && defined(_THREAD_SAFE)) || defined(OSF1)^M
#define PR_NETDB_BUF_SIZE sizeof(struct protoent_data)^M
#else^M
#define PR_NETDB_BUF_SIZE 1024^M
#endif^M
^M
/***********************************************************************^M
** FUNCTION:	^M
** DESCRIPTION:	PR_GetHostByName()^M
** Lookup a host by name.^M
**^M
** INPUTS:^M
**  char *hostname      Character string defining the host name of interest^M
**  char *buf           A scratch buffer for the runtime to return result.^M
**                      This buffer is allocated by the caller.^M
**  PRIntn bufsize      Number of bytes in 'buf'. A recommnded value to^M
**                      use is PR_NETDB_BUF_SIZE.^M
** OUTPUTS:^M
**  PRHostEnt *hostentry^M
**                      This structure is filled in by the runtime if^M
**                      the function returns PR_SUCCESS. This structure^M
**                      is allocated by the caller.^M
** RETURN:^M
**  PRStatus            PR_SUCCESS if the lookup succeeds. If it fails^M
**                      the result will be PR_FAILURE and the reason^M
**                      for the failure can be retrieved by PR_GetError().^M
***********************************************************************/^M
NSPR_API(PRStatus) PR_GetHostByName(^M
    const char *hostname, char *buf, PRIntn bufsize, PRHostEnt *hostentry);^M
^M
/***********************************************************************^M
** FUNCTION:	^M
** DESCRIPTION:	PR_GetIPNodeByName()^M
** Lookup a host by name. Equivalent to getipnodebyname(AI_DEFAULT)^M
** of RFC 2553.^M
**^M
** INPUTS:^M
**  char *hostname      Character string defining the host name of interest^M
**  PRUint16 af         Address family (either PR_AF_INET or PR_AF_INET6)^M
**  PRIntn flags        Specifies the types of addresses that are searched^M
**                      for and the types of addresses that are returned.^M
**                      The only supported flag is PR_AI_DEFAULT.^M
**  char *buf           A scratch buffer for the runtime to return result.^M
**                      This buffer is allocated by the caller.^M
**  PRIntn bufsize      Number of bytes in 'buf'. A recommnded value to^M
**                      use is PR_NETDB_BUF_SIZE.^M
** OUTPUTS:^M
**  PRHostEnt *hostentry^M
**                      This structure is filled in by the runtime if^M
**                      the function returns PR_SUCCESS. This structure^M
**                      is allocated by the caller.^M
** RETURN:^M
**  PRStatus            PR_SUCCESS if the lookup succeeds. If it fails^M
**                      the result will be PR_FAILURE and the reason^M
**                      for the failure can be retrieved by PR_GetError().^M
***********************************************************************/^M
^M
^M
#define PR_AI_ALL        0x08^M
#define PR_AI_V4MAPPED   0x10^M
#define PR_AI_ADDRCONFIG 0x20^M
#define PR_AI_DEFAULT    (PR_AI_V4MAPPED | PR_AI_ADDRCONFIG)^M
^M
NSPR_API(PRStatus) PR_GetIPNodeByName(^M
    const char *hostname,^M
    PRUint16 af,^M
    PRIntn flags,^M
    char *buf,^M
    PRIntn bufsize,^M
    PRHostEnt *hostentry);^M
^M
/***********************************************************************^M
** FUNCTION:	^M
** DESCRIPTION:	PR_GetHostByAddr()^M
** Lookup a host entry by its network address.^M
**^M
** INPUTS:^M
**  char *hostaddr      IP address of host in question^M
**  char *buf           A scratch buffer for the runtime to return result.^M
**                      This buffer is allocated by the caller.^M
**  PRIntn bufsize      Number of bytes in 'buf'. A recommnded value to^M
**                      use is PR_NETDB_BUF_SIZE.^M
** OUTPUTS:^M
**  PRHostEnt *hostentry^M
**                      This structure is filled in by the runtime if^M
**                      the function returns PR_SUCCESS. This structure^M
**                      is allocated by the caller.^M
** RETURN:^M
**  PRStatus            PR_SUCCESS if the lookup succeeds. If it fails^M
**                      the result will be PR_FAILURE and the reason^M
**                      for the failure can be retrieved by PR_GetError().^M
***********************************************************************/^M
NSPR_API(PRStatus) PR_GetHostByAddr(^M
    const PRNetAddr *hostaddr, char *buf, PRIntn bufsize, PRHostEnt *hostentry);^M
^M
/***********************************************************************^M
** FUNCTION:	PR_EnumerateHostEnt()	^M
** DESCRIPTION:^M
**  A stateless enumerator over a PRHostEnt structure acquired from^M
**  PR_GetHostByName() PR_GetHostByAddr() to evaluate the possible^M
**  network addresses.^M
**^M
** INPUTS:^M
**  PRIntn  enumIndex   Index of the enumeration. The enumeration starts^M
**                      and ends with a value of zero.^M
**^M
**  PRHostEnt *hostEnt  A pointer to a host entry struct that was^M
**                      previously returned by PR_GetHostByName() or^M
**                      PR_GetHostByAddr().^M
**^M
**  PRUint16 port       The port number to be assigned as part of the^M
**                      PRNetAddr.^M
**^M
** OUTPUTS:^M
**  PRNetAddr *address  A pointer to an address structure that will be^M
**                      filled in by the call to the enumeration if the^M
**                      result of the call is greater than zero.^M
**^M
** RETURN:^M
**  PRIntn              The value that should be used for the next call^M
**                      of the enumerator ('enumIndex'). The enumeration^M
**                      is ended if this value is returned zero.^M
**                      If a value of -1 is returned, the enumeration^M
**                      has failed. The reason for the failure can be^M
**                      retrieved by calling PR_GetError().^M
***********************************************************************/^M
NSPR_API(PRIntn) PR_EnumerateHostEnt(^M
    PRIntn enumIndex, const PRHostEnt *hostEnt, PRUint16 port, PRNetAddr *address);^M
^M
/***********************************************************************^M
** FUNCTION: PR_InitializeNetAddr(), ^M
** DESCRIPTION:^M
**  Initialize the fields of a PRNetAddr, assigning well known values as^M
**  appropriate.^M
**^M
** INPUTS^M
**  PRNetAddrValue val  The value to be assigned to the IP Address portion^M
**                      of the network address. This can only specify the^M
**                      special well known values that are equivalent to^M
**                      INADDR_ANY and INADDR_LOOPBACK.^M
**^M
**  PRUint16 port       The port number to be assigned in the structure.^M
**^M
** OUTPUTS:^M
**  PRNetAddr *addr     The address to be manipulated.^M
**^M
** RETURN:^M
**  PRStatus            To indicate success or failure. If the latter, the^M
**                      reason for the failure can be retrieved by calling^M
**                      PR_GetError();^M
***********************************************************************/^M
typedef enum PRNetAddrValue^M
{^M
    PR_IpAddrNull,      /* do NOT overwrite the IP address */^M
    PR_IpAddrAny,       /* assign logical INADDR_ANY to IP address */^M
    PR_IpAddrLoopback,  /* assign logical INADDR_LOOPBACK  */^M
    PR_IpAddrV4Mapped   /* IPv4 mapped address */^M
} PRNetAddrValue;^M
^M
NSPR_API(PRStatus) PR_InitializeNetAddr(^M
    PRNetAddrValue val, PRUint16 port, PRNetAddr *addr);^M
^M
/***********************************************************************^M
** FUNCTION: PR_SetNetAddr(), ^M
** DESCRIPTION:^M
**  Set the fields of a PRNetAddr, assigning well known values as^M
**  appropriate. This function is similar to PR_InitializeNetAddr^M
**  but differs in that the address family is specified.^M
**^M
** INPUTS^M
**  PRNetAddrValue val  The value to be assigned to the IP Address portion^M
**                      of the network address. This can only specify the^M
**                      special well known values that are equivalent to^M
**                      INADDR_ANY and INADDR_LOOPBACK.^M
**^M
**  PRUint16 af         The address family (either PR_AF_INET or PR_AF_INET6)^M
**^M
**  PRUint16 port       The port number to be assigned in the structure.^M
**^M
** OUTPUTS:^M
**  PRNetAddr *addr     The address to be manipulated.^M
**^M
** RETURN:^M
**  PRStatus            To indicate success or failure. If the latter, the^M
**                      reason for the failure can be retrieved by calling^M
**                      PR_GetError();^M
***********************************************************************/^M
NSPR_API(PRStatus) PR_SetNetAddr(^M
    PRNetAddrValue val, PRUint16 af, PRUint16 port, PRNetAddr *addr);^M
^M
/***********************************************************************^M
** FUNCTION:	^M
** DESCRIPTION:	PR_IsNetAddrType()^M
** Determine if the network address is of the specified type.^M
**^M
** INPUTS:^M
**  const PRNetAddr *addr   A network address.^M
**  PRNetAddrValue          The type of network address ^M
**^M
** RETURN:^M
**  PRBool                  PR_TRUE if the network address is of the^M
**                          specified type, else PR_FALSE.^M
***********************************************************************/^M
NSPR_API(PRBool) PR_IsNetAddrType(const PRNetAddr *addr, PRNetAddrValue val);^M
^M
/***********************************************************************^M
** FUNCTION:	^M
** DESCRIPTION:	PR_ConvertIPv4AddrToIPv6()^M
** Convert an IPv4 addr to an (IPv4-mapped) IPv6 addr^M
**^M
** INPUTS:^M
**  PRUint32 	v4addr		IPv4 address^M
**^M
** OUTPUTS:^M
**  PRIPv6Addr *v6addr      The converted IPv6 address^M
**^M
** RETURN:^M
**  void^M
**                       ^M
***********************************************************************/^M
NSPR_API(void) PR_ConvertIPv4AddrToIPv6(PRUint32 v4addr, PRIPv6Addr *v6addr);^M
^M
/***********************************************************************^M
** MACRO:	^M
** DESCRIPTION:	PR_NetAddrFamily()^M
** Get the 'family' field of a PRNetAddr union.^M
**^M
** INPUTS:^M
**  const PRNetAddr *addr   A network address.^M
**^M
** RETURN:^M
**  PRUint16                The 'family' field of 'addr'.^M
***********************************************************************/^M
#define PR_NetAddrFamily(addr) ((addr)->raw.family)^M
^M
/***********************************************************************^M
** MACRO:	^M
** DESCRIPTION:	PR_NetAddrInetPort()^M
** Get the 'port' field of a PRNetAddr union.^M
**^M
** INPUTS:^M
**  const PRNetAddr *addr   A network address.^M
**^M
** RETURN:^M
**  PRUint16                The 'port' field of 'addr'.^M
***********************************************************************/^M
#define PR_NetAddrInetPort(addr) \^M
    ((addr)->raw.family == PR_AF_INET6 ? (addr)->ipv6.port : (addr)->inet.port)^M
^M
/***********************************************************************^M
** FUNCTION:	^M
** DESCRIPTION:	PR_GetProtoByName()^M
** Lookup a protocol entry based on protocol's name^M
**^M
** INPUTS:^M
**  char *protocolname  Character string of the protocol's name.^M
**  char *buf           A scratch buffer for the runtime to return result.^M
**                      This buffer is allocated by the caller.^M
**  PRIntn bufsize      Number of bytes in 'buf'. A recommnded value to^M
**                      use is PR_NETDB_BUF_SIZE.^M
** OUTPUTS:^M
**  PRHostEnt *PRProtoEnt^M
**                      This structure is filled in by the runtime if^M
**                      the function returns PR_SUCCESS. This structure^M
**                      is allocated by the caller.^M
** RETURN:^M
**  PRStatus            PR_SUCCESS if the lookup succeeds. If it fails^M
**                      the result will be PR_FAILURE and the reason^M
**                      for the failure can be retrieved by PR_GetError().^M
***********************************************************************/^M
^M
typedef struct PRProtoEnt {^M
    char *p_name;       /* official protocol name */^M
    char **p_aliases;   /* alias list */^M
#if defined(WIN32) || defined(WIN16)^M
    PRInt16 p_num;      /* protocol # */^M
#else^M
    PRInt32 p_num;      /* protocol # */^M
#endif^M
} PRProtoEnt;^M
^M
NSPR_API(PRStatus) PR_GetProtoByName(^M
    const char* protocolname, char* buffer, PRInt32 bufsize, PRProtoEnt* result);^M
^M
/***********************************************************************^M
** FUNCTION:	^M
** DESCRIPTION:	PR_GetProtoByNumber()^M
** Lookup a protocol entry based on protocol's number^M
**^M
** INPUTS:^M
**  PRInt32 protocolnumber^M
**                      Number assigned to the protocol.^M
**  char *buf           A scratch buffer for the runtime to return result.^M
**                      This buffer is allocated by the caller.^M
**  PRIntn bufsize      Number of bytes in 'buf'. A recommnded value to^M
**                      use is PR_NETDB_BUF_SIZE.^M
** OUTPUTS:^M
**  PRHostEnt *PRProtoEnt^M
**                      This structure is filled in by the runtime if^M
**                      the function returns PR_SUCCESS. This structure^M
**                      is allocated by the caller.^M
** RETURN:^M
**  PRStatus            PR_SUCCESS if the lookup succeeds. If it fails^M
**                      the result will be PR_FAILURE and the reason^M
**                      for the failure can be retrieved by PR_GetError().^M
***********************************************************************/^M
NSPR_API(PRStatus) PR_GetProtoByNumber(^M
    PRInt32 protocolnumber, char* buffer, PRInt32 bufsize, PRProtoEnt* result);^M
^M
/***********************************************************************^M
** FUNCTIONS: PR_ntohs, PR_ntohl, PR_ntohll, PR_htons, PR_htonl, PR_htonll^M
**^M
** DESCRIPTION: API entries for the common byte ordering routines.^M
**^M
**      PR_ntohs        16 bit conversion from network to host^M
**      PR_ntohl        32 bit conversion from network to host^M
**      PR_ntohll       64 bit conversion from network to host^M
**      PR_htons        16 bit conversion from host to network^M
**      PR_htonl        32 bit conversion from host to network^M
**      PR_ntonll       64 bit conversion from host to network^M
**^M
***********************************************************************/^M
NSPR_API(PRUint16) PR_ntohs(PRUint16);^M
NSPR_API(PRUint32) PR_ntohl(PRUint32);^M
NSPR_API(PRUint64) PR_ntohll(PRUint64);^M
NSPR_API(PRUint16) PR_htons(PRUint16);^M
NSPR_API(PRUint32) PR_htonl(PRUint32);^M
NSPR_API(PRUint64) PR_htonll(PRUint64);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prnetdb_h___ */^M
