/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prthread_h___^M
#define prthread_h___^M
^M
/*^M
** API for NSPR threads. On some architectures (MAC and WIN16^M
** notably) pre-emptibility is not guaranteed. Hard priority scheduling^M
** is not guaranteed, so programming using priority based synchronization^M
** is a no-no.^M
**^M
** NSPR threads are scheduled based loosly on their client set priority.^M
** In general, a thread of a higher priority has a statistically better^M
** chance of running relative to threads of lower priority. However,^M
** NSPR uses multiple strategies to provide execution vehicles for thread^M
** abstraction of various host platforms. As it turns out, there is little^M
** NSPR can do to affect the scheduling attributes of "GLOBAL" threads.^M
** However, a semblance of GLOBAL threads is used to implement "LOCAL"^M
** threads. An arbitrary number of such LOCAL threads can be assigned to^M
** a single GLOBAL thread.^M
**^M
** For scheduling, NSPR will attempt to run the highest priority LOCAL^M
** thread associated with a given GLOBAL thread. It is further assumed^M
** that the host OS will apply some form of "fair" scheduling on the^M
** GLOBAL threads.^M
**^M
** Threads have a "system flag" which when set indicates the thread^M
** doesn't count for determining when the process should exit (the^M
** process exits when the last user thread exits).^M
**^M
** Threads also have a "scope flag" which controls whether the threads^M
** are scheduled in the local scope or scheduled by the OS globally. This ^M
** indicates whether a thread is permanently bound to a native OS thread. ^M
** An unbound thread competes for scheduling resources in the same process.^M
**^M
** Another flag is "state flag" which control whether the thread is joinable.^M
** It allows other threads to wait for the created thread to reach completion.^M
**^M
** Threads can have "per-thread-data" attached to them. Each thread has a^M
** per-thread error number and error string which are updated when NSPR^M
** operations fail.^M
*/^M
#include "prtypes.h"^M
#include "prinrval.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
typedef struct PRThread PRThread;^M
typedef struct PRThreadStack PRThreadStack;^M
^M
typedef enum PRThreadType {^M
    PR_USER_THREAD,^M
    PR_SYSTEM_THREAD^M
} PRThreadType;^M
^M
typedef enum PRThreadScope {^M
    PR_LOCAL_THREAD,^M
    PR_GLOBAL_THREAD,^M
    PR_GLOBAL_BOUND_THREAD^M
} PRThreadScope;^M
^M
typedef enum PRThreadState {^M
    PR_JOINABLE_THREAD,^M
    PR_UNJOINABLE_THREAD^M
} PRThreadState;^M
^M
typedef enum PRThreadPriority^M
{^M
    PR_PRIORITY_FIRST = 0,      /* just a placeholder */^M
    PR_PRIORITY_LOW = 0,        /* the lowest possible priority */^M
    PR_PRIORITY_NORMAL = 1,     /* most common expected priority */^M
    PR_PRIORITY_HIGH = 2,       /* slightly more aggressive scheduling */^M
    PR_PRIORITY_URGENT = 3,     /* it does little good to have more than one */^M
    PR_PRIORITY_LAST = 3        /* this is just a placeholder */^M
} PRThreadPriority;^M
^M
/*^M
** Create a new thread:^M
**     "type" is the type of thread to create^M
**     "start(arg)" will be invoked as the threads "main"^M
**     "priority" will be created thread's priority^M
**     "scope" will specify whether the thread is local or global^M
**     "state" will specify whether the thread is joinable or not^M
**     "stackSize" the size of the stack, in bytes. The value can be zero^M
**        and then a machine specific stack size will be chosen.^M
**^M
** This can return NULL if some kind of error occurs, such as if memory is^M
** tight.^M
**^M
** If you want the thread to start up waiting for the creator to do^M
** something, enter a lock before creating the thread and then have the^M
** threads start routine enter and exit the same lock. When you are ready^M
** for the thread to run, exit the lock.^M
**^M
** If you want to detect the completion of the created thread, the thread^M
** should be created joinable.  Then, use PR_JoinThread to synchrnoize the^M
** termination of another thread.^M
**^M
** When the start function returns the thread exits. If it is the last^M
** PR_USER_THREAD to exit then the process exits.^M
*/^M
NSPR_API(PRThread*) PR_CreateThread(PRThreadType type,^M
                     void (*start)(void *arg),^M
                     void *arg,^M
                     PRThreadPriority priority,^M
                     PRThreadScope scope,^M
                     PRThreadState state,^M
                     PRUint32 stackSize);^M
^M
/*^M
** Wait for thread termination:^M
**     "thread" is the target thread ^M
**^M
** This can return PR_FAILURE if no joinable thread could be found ^M
** corresponding to the specified target thread.^M
**^M
** The calling thread is blocked until the target thread completes.^M
** Several threads cannot wait for the same thread to complete; one thread^M
** will operate successfully and others will terminate with an error PR_FAILURE.^M
** The calling thread will not be blocked if the target thread has already^M
** terminated.^M
*/^M
NSPR_API(PRStatus) PR_JoinThread(PRThread *thread);^M
^M
/*^M
** Return the current thread object for the currently running code.^M
** Never returns NULL.^M
*/^M
NSPR_API(PRThread*) PR_GetCurrentThread(void);^M
#define PR_CurrentThread() PR_GetCurrentThread() /* for nspr1.0 compat. */^M
^M
/*^M
** Get the priority of "thread".^M
*/^M
NSPR_API(PRThreadPriority) PR_GetThreadPriority(const PRThread *thread);^M
^M
/*^M
** Change the priority of the "thread" to "priority".^M
*/^M
NSPR_API(void) PR_SetThreadPriority(PRThread *thread, PRThreadPriority priority);^M
^M
/*^M
** This routine returns a new index for per-thread-private data table. ^M
** The index is visible to all threads within a process. This index can ^M
** be used with the PR_SetThreadPrivate() and PR_GetThreadPrivate() routines ^M
** to save and retrieve data associated with the index for a thread.^M
**^M
** Each index is associationed with a destructor function ('dtor'). The function^M
** may be specified as NULL when the index is created. If it is not NULL, the^M
** function will be called when:^M
**      - the thread exits and the private data for the associated index^M
**        is not NULL,^M
**      - new thread private data is set and the current private data is^M
**        not NULL.^M
**^M
** The index independently maintains specific values for each binding thread. ^M
** A thread can only get access to its own thread-specific-data.^M
**^M
** Upon a new index return the value associated with the index for all threads^M
** is NULL, and upon thread creation the value associated with all indices for ^M
** that thread is NULL. ^M
**^M
** Returns PR_FAILURE if the total number of indices will exceed the maximun ^M
** allowed.^M
*/^M
typedef void (PR_CALLBACK *PRThreadPrivateDTOR)(void *priv);^M
^M
NSPR_API(PRStatus) PR_NewThreadPrivateIndex(^M
    PRUintn *newIndex, PRThreadPrivateDTOR destructor);^M
^M
/*^M
** Define some per-thread-private data.^M
**     "tpdIndex" is an index into the per-thread private data table^M
**     "priv" is the per-thread-private data ^M
**^M
** If the per-thread private data table has a previously registered^M
** destructor function and a non-NULL per-thread-private data value,^M
** the destructor function is invoked.^M
**^M
** This can return PR_FAILURE if the index is invalid.^M
*/^M
NSPR_API(PRStatus) PR_SetThreadPrivate(PRUintn tpdIndex, void *priv);^M
^M
/*^M
** Recover the per-thread-private data for the current thread. "tpdIndex" is^M
** the index into the per-thread private data table. ^M
**^M
** The returned value may be NULL which is indistinguishable from an error ^M
** condition.^M
**^M
** A thread can only get access to its own thread-specific-data.^M
*/^M
NSPR_API(void*) PR_GetThreadPrivate(PRUintn tpdIndex);^M
^M
/*^M
** This routine sets the interrupt request for a target thread. The interrupt^M
** request remains in the thread's state until it is delivered exactly once^M
** or explicitly canceled.^M
**^M
** A thread that has been interrupted will fail all NSPR blocking operations^M
** that return a PRStatus (I/O, waiting on a condition, etc).^M
**^M
** PR_Interrupt may itself fail if the target thread is invalid.^M
*/^M
NSPR_API(PRStatus) PR_Interrupt(PRThread *thread);^M
^M
/*^M
** Clear the interrupt request for the calling thread. If no such request^M
** is pending, this operation is a noop.^M
*/^M
NSPR_API(void) PR_ClearInterrupt(void);^M
^M
/*^M
** Block the interrupt for the calling thread.^M
*/^M
NSPR_API(void) PR_BlockInterrupt(void);^M
^M
/*^M
** Unblock the interrupt for the calling thread.^M
*/^M
NSPR_API(void) PR_UnblockInterrupt(void);^M
^M
/*^M
** Make the current thread sleep until "ticks" time amount of time^M
** has expired. If "ticks" is PR_INTERVAL_NO_WAIT then the call is^M
** equivalent to calling PR_Yield. Calling PR_Sleep with an argument^M
** equivalent to PR_INTERVAL_NO_TIMEOUT is an error and will result^M
** in a PR_FAILURE error return.^M
*/^M
NSPR_API(PRStatus) PR_Sleep(PRIntervalTime ticks);^M
^M
/*^M
** Get the scoping of this thread.^M
*/^M
NSPR_API(PRThreadScope) PR_GetThreadScope(const PRThread *thread);^M
^M
/*^M
** Get the type of this thread.^M
*/^M
NSPR_API(PRThreadType) PR_GetThreadType(const PRThread *thread);^M
^M
/*^M
** Get the join state of this thread.^M
*/^M
NSPR_API(PRThreadState) PR_GetThreadState(const PRThread *thread);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prthread_h___ */^M
